// achievements.js
// This model for tracking achievements is not the only possible one,
// but it should be useful for many situations.
//
// There are two levels of achievements. There are a top level of achievements
// but each achievement can consist of a set of sub-achievements.
// We can use the badges module to determine which badges to display for the current
// set of completed achievements.
// When a set of sub-achievements completed for a given top level achievement, a "medal"
// is displayed for each achievement. When the entire set of sub-achievements is completed,
// the medals are replaced with a trophy.

import $ from "./jquery.module.js";
import SetConfig from "./set_config_class.js";
import * as badges from "./badges.js";

const achievementsKey = "achievementJSON";
let setConfigHandler = new SetConfig();
let config = null;
let achievements = {};

function setConfig(config) {
    setConfigHandler.setConfig(config);
}

function addCompletedAchievement({achievement, subAchievement}) {
    if (!(achievement in achievements)) {
        achievements[achievement] = {};
    }
    let subAchievements = achievements[achievement];
    subAchievements[subAchievement] = true;
    localStorage.setItem(achievementsKey, JSON.stringify(achievements));

    // Update badges
    const goals = config.achievements[achievement];
    const maxGoals = goals.length;
    let badgeCount = 0;
    for (const subGoal of goals) {
        if (subGoal in subAchievements && subAchievements[subGoal]) {
            badgeCount += 1;
        }
    }
    if (badgeCount > 0 && badgeCount < maxGoals) {
        badges.setBadge({activity: achievement, badge: "medal", count: badgeCount});
    }
    else if (badgeCount === maxGoals) {
        badges.setBadge({activity: achievement, badge: "trophy", count: 1});
    }
    // Add a badge for the sub-achievement itself
    badges.setBadge({activity: subAchievement, badge: "medal", count: 1});
}

function clearAchievements() {
    achievements = {};
    localStorage.setItem(achievementsKey, JSON.stringify(achievements));
}

$(async () => {
    config = await setConfigHandler.getConfigAsync();
    let achievementJSON = localStorage.getItem(achievementsKey);
    if (achievementJSON == null) {
        clearAchievements();
    }
    else {
        try {
            achievements = JSON.parse(achievementJSON);
        }
        catch(error) {
            console.log(error);
            clearAchievements();
        }
    }
});

export {setConfig, addCompletedAchievement, clearAchievements};