// Module gamestate
// exports getGameState, updateGameState

import * as db from "./database_access.js";
import * as user from "./username.js";
import * as config from "./gamestate_config.js";

let gamestatedata = null;

let pointHandlers = []; // pointHandlers is an array of handlers that get called when the points
                        // are set to a new value via setPoints

function getGameState() {
    if (gamestatedata == null) {
        let jsonData = sessionStorage.getItem(config.gameStateKey);
        if (jsonData) {
            gamestatedata = JSON.parse(jsonData);
        }
        else {
            gamestatedata = {};
        }
    }

    return gamestatedata;
}

// function updateGameState
// This function updates the database on the server. If called from a page that
// can potentially navigate away from the page before the server update is complete,
// the page needs to wait until successful completion. This can be done either
//
//    1) By delivering a completion routine, i.e.
//          updateGameState(function(data) {window.location.href = "newpage,html";}
//
//    2) ...or by using the promise returned by updateGameState, i.e.
//          updateGameState().then( () => window.location.href = "newpage.html" );
//       Using the promise, the navigation can be deferred to another part of the code,
//       which may be more convienent:
//          var updatePromise = updateGameState();
//          .
//          .
//          .
//          [do some stuff]
//          .
//          .
//          .
//          updatePromise.then( () => window.location.href = "newpage.html" );

function updateGameState(completion_routine, error_continue_routine) {
    let continue_routine = null;
    if (typeof(error_continue_routine) == "function") {
        continue_routine = error_continue_routine;
    }

    let g = getGameState();
    let username = user.getUserName();

    let jsonData = JSON.stringify(g);
    sessionStorage.setItem(config.gameStateKey, jsonData);
    completion_routine = completion_routine || null; // completion_routine is optional
    let data = {
        action:'save-db-state',
        userid: username,
        jsondata: jsonData
    };

    // Add any fields in the gamestate record in addition to jsondata;
    // typically these values are cached in sessionStorage
    for (const fieldEntry of config.cached_db_fields) {
        const field = fieldEntry.field;
        data[field] = sessionStorage.getItem(fieldEntry.storageKey);
    }

    let dbPromise = db.perform_db_action_promise(data);

    if (typeof(completion_routine) === "function" ) {
        dbPromise.done(completion_routine);
    }

    if (typeof(error_continue_routine) === "function") {
        dbPromise.fail(error_continue_routine);
    }

    return dbPromise;

}

// Game-dependent convenience routines
// Note: in the actual game the sessionStorage caches will always be set up correctly,
// so they will never return null, but we may be testing in the browser, so we check for
// null so we can return a sane value in that case.

function getPoints() {
    let points = sessionStorage.getItem(config.pointsKey);
    return points === null ? 0 : Number(points);
}

function setPoints(points) {
    sessionStorage.setItem(config.pointsKey, points.toString());
    updateGameState();
    for (const h of pointHandlers) {
        h(points);
    }
}

function addPoints(points) {
    setPoints(getPoints() + Number(points));
}

function addPointHandler(h) {
    pointHandlers.push(h);
}

function getSession() {
    let session = sessionStorage.getItem(config.sessionKey);
    return session === null ? 1 : session;
}

function setSession(sessionNumber) {
    sessionStorage.setItem(config.sessionKey, sessionNumber);
    updateGameState();
}

function initializeGameLogDataForNewUser() {
    gamestatedata = {};
    sessionStorage.setItem(config.pointsKey, 0);
    sessionStorage.setItem(config.sessionKey, 1);
    updateGameState();
}

function setGameStateFromJSON(items, callback) {
    sessionStorage.setItem(config.gameStateKey, items);
    if (items !== "") {
        const gameStateValues = JSON.parse(items);
        let g = getGameState();
        for (const [key, value] of Object.entries(gameStateValues)) {
            g[key] = value;
        }
    }
    else {
        if (callback) {
            callback();
        }
    }
}


export {getGameState, updateGameState, initializeGameLogDataForNewUser, setGameStateFromJSON, getSession, setSession};
export {getPoints as points, setPoints, addPoints, addPointHandler};
export {safe_go_to, safe_reload_page} from "./database_access.js";

