// inventory.js module
// The inventory is an object stored in the gamestate and is of the form:
// inventory = {
//              item1: {count: n1,
//                      otherProp: value},
//              item2: {count: n2,
//                      otherProp: value}
//              }
//
// The possible values allowed for item[n] are the keys in the inventory_item_attributes object
// in the inventory_config.js file. The properties allowed for each item in the inventory is up
// to the developer. The only required property is "count". Attributes of the item that remain constant
// for that item are in inventory_item_attributes.


import * as config    from "./inventory_config.js";
import * as user      from "./username.js";
import * as gamestate from "./gamestate.js";
import * as test_inventory from "./test_inventory.js";

let inventory = null;

if (test_inventory.use_test_inventory)
{
    let g = gamestate.getGameState();
    if (!("inventory" in g)) {
        g.inventory = test_inventory.test_inventory();
        gamestate.updateGameState();
    }
    inventory = g.inventory;
}


function getInventory() {
    if (inventory === null) {
        let g = gamestate.getGameState();

        if (!("inventory" in g)) {
            g.inventory = {};
            gamestate.updateGameState();
        }

        inventory = g.inventory;
    }

    return inventory;
}

function addInventoryItem(itemName) {
    let inventory = getInventory();

    if (itemName in inventory) {
        let itemProperties = inventory[itemName];
        itemProperties.count++;
    }
    else {
        let itemProperties = {...config.initial_item_property_values};

        inventory[itemName] = itemProperties;
    }

    gamestate.updateGameState();
}

function itemAttributes() {
    return config.inventory_item_attributes;
}

function attributesForItem(itemName) {
    if (itemName in config.inventory_item_attributes) {
        return config.inventory_item_attributes[itemName];
    }
    else {
        return null;
    }
}

function itemCount(itemName) {
    let inventory = getInventory();
    if (itemName in inventory) {
        return inventory[itemName].count;
    }
    else {
        return 0;
    }
}

function addMultipleItems(itemName, numItems) {
    let inventory = getInventory();
    if (itemName in inventory) {
        let item = inventory[itemName];
        item.count = item.count + numItems;
        gamestate.updateGameState();
    }
}

function updateDB() {
    // If the inventory object is changed outside of the functions in these modules, the user
    // calls this to write the changes to the database. All this does is call updateGameState,
    // but it is provided as a convenience so that the user isn't forced to import the
    // gamestate module.

    gamestate.updateGameState();
}

function filterEntriesByAttribute(attribute, value, inventory = getInventory()) {
    // Gets the "entries" array of the inventory object and then filters
    // that array by the property of the item
    // The optional third argument makes it possible to use a different inventory object than
    // the actual inventory; this is provided primarily for testing purposes

    return Object.entries(inventory).filter(entry => config.inventory_item_attributes[entry[0]][attribute] === value);
}

export {getInventory, addInventoryItem, itemCount, attributesForItem, addMultipleItems, updateDB, filterEntriesByAttribute};
export {safe_go_to} from "./gamestate.js";