import $ from "./jquery.module.js";
import SetConfig from "./set_config_class.js";
import {getRandomInt} from "./random.js";
import {showDialogAsync, showChoiceDialogAsync} from "./dialog.js";

let setConfigHandler = new SetConfig();
let config;
let options;
let solutionHandler = null;
let startChoices = {
    rank: 3,
    difficulty: "hard",
};

const randomizerSteps = 30;
const hintClass = "lights-out-hint";

function setSolutionHandler(f) {
    solutionHandler = f;
}

let numberSolved = 0;

async function solvedPuzzle(gameBoard) {
    $(`#${gameBoard.boardID} .lights-out-message`).text("You solved this puzzle!!!");
    numberSolved++;
    if (numberSolved >= 4) {
        await showDialogAsync("You solved all four puzzles!!");
        $("#grid-container").addClass("d-none");
        $("#wrapper").css("background-image", "none");
        const choice = await showChoiceDialogAsync("Do you want to play more Lights Out?", ["Play Lights Out Again", "Continue"]);
        if (choice === "Continue") {
            window.location.href = config.options.completionURL;
        }
        else {
            $("#lights-out-intro").removeClass("d-none");
            $("#grid-container").addClass("d-none");
        }
    }
}

setSolutionHandler(solvedPuzzle); // This can be overridden by module user with setSolutionHandler

$.fn.setLight = function(light) {
    switch (light) {
        case "on":
            this
                .removeClass("lights-off-class")
                .addClass("lights-on-class")
                .css({
                    "background-image": `url("${options.lightsOnImg}")`,
                    "background-size": "contain",
                });

            break;

        case "off":
        case "out":
            this
                .removeClass("lights-on-class")
                .addClass("lights-off-class")
                .css({
                    "background-image": `url("${options.lightsOutImg}")`,
                    "background-size": "contain",
                });
            break;
    }

    return this;
}

function setConfig(config) {
    setConfigHandler.setConfig(config);
}

let gridItems = [];
let numberOfMoves = 0;

function toggleLit(gridItem) {
    gridItem.lit = !gridItem.lit;
    const $item = gridItem.$item;
    switch (gridItem.lit) {
        case false:
            $item.setLight("off");
            break;

        case true:
            $item.setLight("on");
            break;
    }
}

function toggleGridItem({row, col}) {
    if (row < 0 || row >= this.gridItems.length) {
        return;
    }
    const gridRow = this.gridItems[row];
    if (col < 0 || col >= gridRow.length) {
        return;
    }
    toggleLit(gridRow[col]);
}

function setGridItem({row, col, lit}) {
    if (row < 0 || row >= this.gridItems.length) {
        return;
    }
    const gridRow = this.gridItems[row];
    if (col < 0 || col >= gridRow.length) {
        return;
    }

    const gridItem = gridRow[col];

    switch (lit) {
        case false:
            gridItem.$item.setLight("off");
            break;

        case true:
            gridItem.$item.setLight("on");
            break;
    }
    gridItem.lit = lit;
}

function clearGameGrid() {
    // Set to "solved" state
    const lit = false;
    const rank = this.rank;
    for (let row = 0; row < rank; row++) {
        for (let col = 0; col < rank; col++) {
            this.setGridItem({row, col, lit});
        }
    }
}

async function checkForSolution() {
    let solved = true;
    topLoop:
    for (const gridRow of this.gridItems) {
        for (const gridItem of gridRow) {
            if (gridItem.lit) { // Then not solved
                solved  = false;
                break topLoop;
            }
        }
    }

    if (solved) {
        this.solved = true;
        await showDialogAsync(`Congratulations! You solved this puzzle in ${this.numberOfMoves} moves!`);
        if (solutionHandler) {
            solutionHandler(this);
        }
        else {
            window.location.href = config.options.completionURL;
        }
    }

    return solved;
}

async function clickGridItem(event) {
    const {row, col, $gameGrid} = event.data;
    const gameBoard = $gameGrid.data("gameBoard");
    if (gameBoard.solved) {
        return; // Don't respond to clicks on solved puzzle
    }
    gameBoard.toggleItemAndNeighbors({row, col});
    gameBoard.numberOfMoves++;
    gameBoard.showHints(); // If show hints is active, this updates the hints display

    let solved = await gameBoard.checkForSolution();
}

async function clickHintCheckbox(event) {
    const gameBoard = event.data.gameBoard;
    gameBoard.showingHints = event.currentTarget.checked;
    gameBoard.showHints();
    // console.log(`Clicked checkbox for game board ${gameBoard.boardID}, checkbox is ${checked}`);
}

function toggleItemAndNeighbors({row, col}) {
    this.toggleGridItem({row, col});
    this.toggleGridItem({row: row - 1, col});
    this.toggleGridItem({row: row + 1, col});
    this.toggleGridItem({row, col: col - 1});
    this.toggleGridItem({row, col: col + 1});
    const toggle =  !this.solution[row][col];
    this.solution[row][col] = toggle;
    if (toggle) {
        this.solution.steps += 1;
    }
    else {
        this.solution.steps -= 1;
    }
}

function generateRandomPuzzle() {
    // Note: if difficulty is set to "easy" then the random generation stops when the
    // number of steps in the solution reaches the rank of the puzzle.

    this.clearGameGrid(); // Start at "solved" state: all lights out
    const easy = (this.difficulty === "easy");
    const rank = this.rank;
    for (let i = 0; i < randomizerSteps; i++) {
        const row = getRandomInt(this.rank);
        const col = getRandomInt(this.rank);
        this.toggleItemAndNeighbors({row, col});
        if (easy && this.solution.steps >= rank) {
            break;
        }
    }
    $(".lights-on-class")
}

function makeSolutionStructure(rank) {
    // Returns a rank x rank matrix with all entries marked false
    let solution = [];
    for(let row = 0; row < rank; row++) {
        let solutionRow = [];
        for (let col = 0; col < rank; col++) {
            solutionRow.push(false);
        }
        solution.push(solutionRow);
    }

    // Keep a count of the number of steps inn the solution
    solution.steps = 0;
    return solution;
}

function showHints() {
    if (this.showingHints) {
        const rank = this.rank;
        const solution = this.solution;
        for (let row = 0; row < rank; row++) {
            const boardRow = this.gridItems[row];
            for (let col = 0; col < rank; col++) {
                const gridItem = boardRow[col];
                if (solution[row][col]) {
                    gridItem.$item.addClass(hintClass);
                }
                else {
                    gridItem.$item.removeClass(hintClass);
                }
            }
        }
    }
    else {
        this.$gameGrid.children().removeClass(hintClass);
    }
}

function generateGameGrid({rank, difficulty}) {
    // Returns matrix with display items and top level div
    let $gameGrid =  $(`<div class="game-grid"></div>`).data("rank", rank);
    $gameGrid.css("grid-template-columns", `repeat(${rank}, 1fr)`);
    $gameGrid.css("grid-template-rows", `repeat(${rank}, 1fr)`);
    let gridItems = [];
    for (let row = 0; row < rank; row++) {
        gridItems[row] = [];
        for (let col = 0; col < rank; col++) {
            const $item = $(`<div id="item-${row}-${col}" class="lights-off-class"></div>`);
            $gameGrid.append($item);
            $item.on("click", {row, col, $gameGrid}, clickGridItem);
            gridItems[row][col] = {$item, row, col, lit: false};
        }
    }
    const solution = makeSolutionStructure(rank);

    const gameBoard = {
        $gameGrid,
        gridItems,
        rank,
        difficulty,
        setGridItem,
        generateRandomPuzzle,
        clearGameGrid,
        toggleGridItem,
        toggleItemAndNeighbors,
        checkForSolution,
        showHints,
        solution,
        numberOfMoves: 0,
        solved: false,
        showingHints: false,
    }
    $gameGrid.data("gameBoard", gameBoard);
    return gameBoard;
}

function startGame(event) {
    numberSolved = 0;
    $("#lights-out-intro").addClass("d-none");
    $("#grid-container").removeClass("d-none");

    $("#wrapper").css("background-image", `url("${options.backgroundImg}")`);

    startChoices = event.data;
    let rank = startChoices?.rank ?? (options?.gameBoardRank ?? 3);
    let difficulty = startChoices?.difficulty ?? "hard";

    // create grid
    for (const boardID of options.gameBoards) {
        const $gameGrid = $(`#${boardID} .game-board`);
        $gameGrid.html(""); // Clear any previous contents
        const gameBoard = generateGameGrid({rank, difficulty});
        gameBoard.generateRandomPuzzle();
        gameBoard.boardID = boardID;
        $gameGrid.append(gameBoard.$gameGrid);
        const $hintCheckbox = $(`#${boardID} .hint-checkbox`);
        $hintCheckbox.on("click", {gameBoard}, clickHintCheckbox);

        // Clear some things that might have been set if we are replaying
        $hintCheckbox[0].checked = false;
    }

    // Clear some things that might have been set if we are replaying
    $(".lights-out-message").html("");
}

async function showHintText(event) {
    await showDialogAsync(options.hintText);
}

$(async () => {
    config = await setConfigHandler.getConfigAsync();
    options = config.options;

    $("#lights-out-intro")
        .html(options.introText)
        .append(`
        <div id="lights-out-start-game-buttons">
        <button id='play-game-rank-3'>Play Game 3x3</button>
        <button id='play-game-rank-3-easy'>Play Game 3x3 / Easy Mode</button>
        <button id='play-game-rank-5'>Play Game 5x5</button>
        <button id='play-game-rank-5-easy'>Play Game 5x5 / Easy Mode</button>
        </div>
        `);

    $("#play-game-rank-3").on("click", {rank: 3, difficulty: "hard"}, startGame);
    $("#play-game-rank-3-easy").on("click", {rank: 3, difficulty: "easy"}, startGame);
    $("#play-game-rank-5").on("click", {rank: 5, difficulty: "hard"}, startGame);
    $("#play-game-rank-5-easy").on("click", {rank: 5, difficulty: "easy"}, startGame);
    $("#hint-btn").on("click", showHintText)

});

export {setConfig, setSolutionHandler};
