// login.js
// module

import $ from "./jquery.module.js";
import SetConfig from "./set_config_class.js";
import * as user      from "./username.js";
import * as db        from "./database.js";
import * as gamestate from "./gamestate.js";
import * as offline   from "./offline_play.js";
import * as milestone from "./milestone.js";
import * as home      from "./home.js";
import * as dialog from "./dialog.js";

let setConfigHandler = new SetConfig();

let loginCompleteCallback = null;

let login = null;

function setConfig(config) {
    setConfigHandler.setConfig(config);
}

function setLoginCompleteCallback(f) {
    loginCompleteCallback = f;
}

function showForm(formID) {
    let forms = Object.entries(login.forms);
    for (const [, ID] of forms) {
        if (ID === formID) {
            $(ID).show();
        }
        else {
            $(ID).hide();
        }
    }
}

function showFormWithEventData(evt) {
    if (evt.data && evt.data.showForm) {showForm(evt.data.showForm);}
    evt.preventDefault();
}

async function user_lookup(theUser){
    let lookupNotComplete = true;
    let lookupResult = null;
    while (lookupNotComplete) {
        let transactionResult = await db.perform_db_transaction(
            {
                action:'lookup-userid',
                userid: theUser
            }
        );

        if (transactionResult.transactionWorked) {
            lookupNotComplete = false;
            lookupResult = transactionResult;
        }
        else {
            await retryDialog;
        }
    }

    return lookupResult;
}

async function user_login(theUser, password, completionRoutine){
    let lookupNotComplete = true;
    let lookupResult = null;
    while (lookupNotComplete) {
        let transactionResult = await db.perform_db_transaction(
            {
                action:'user-login',
                userid: theUser,
                password: password
            }
        );

        if (transactionResult.transactionWorked) {
            lookupNotComplete = false;
            lookupResult = transactionResult.result;
        }
        else {
            await retryDialog;
        }
    }

    return lookupResult;

}


function extractGameStateData(data) {
    // Save JSON for gamestate in session storage,
    // and also extract some game state components.
    let jsondata = data.jsondata;
    if (jsondata) {
        gamestate.setGameStateFromJSON(jsondata);
    }
    else {
        gamestate.setGameStateFromJSON("");
    }

    if ("points" in data && data.points != null) {
        gamestate.setPoints(data.points);
    }
    else {
        gamestate.setPoints(0);
    }

    if ("session_count" in data && data.session_count != null) {
        gamestate.setSession(data.session_count);
    }
    else {
        gamestate.setSession(1);
    }
}

function playOffline(){
    showForm(login.forms.offline);
    return false;
}

function offlineNewGame() {
    offline.setOfflinePlay();
    if (login.options.handleGameState) {
        gamestate.setGameStateFromJSON("{}");
    }
    showForm(login.forms.success);
    return false;
}

function offlineReturningPlayer() {
    offline.setOfflinePlay();
    if (login.options.handleGameState) {
        let jsonData = localStorage.getItem(localGameStateKey);
        extractGameStateData(jsonData);
    }
    showForm(login.forms.success);
    return false;
}

function clearUserNameFields() {
    $(login.elements.oldUsername).val("");
    $(login.elements.newUsername).val("");
}

function clearPasswordFields(args) {
    $(login.elements.newPassword).val("");
    $(login.elements.verifyPassword).val("");
    $(login.elements.oldPassword).val("");
    showForm(args.data.showForm);
}

function refreshLoginForm() {
    clearUserNameFields()
    clearPasswordFields();
    showForm(login.forms.initial);
    return false;
}


async function submitUserAndPassword(evt) {
    evt.preventDefault();
    const username = $(login.elements.oldUsername).val();
    user.setUserName(username);
    let transactionResult = await user_lookup(username);
    let message = transactionResult.result.data.message;
    if (message === 'New password required') {
        showForm(login.forms.nameNotFound);
    }
    else {
        await submitExistingPassword();
    }

}

async function submitExistingPassword() {
    let username = user.getUserName();
    let password = $(login.elements.oldPassword).val();
    if (password) {
        let result = await user_login(username, password);
        let data = result?.data;
        if (!data || data.message === "Password did not match") {
            showForm(login.forms.wrongPassword)
        }
        else {
            if (login.options.handleGameState) {
                extractGameStateData(data);
            }
            showForm(login.forms.success);
            if (loginCompleteCallback !== null) {
                loginCompleteCallback();
            }
        }
    }
}

async function submitNewUserAndPassword(evt) {
    evt.preventDefault();
    const username = $(login.elements.newUsername).val();
    user.setUserName(username);
    let transactionResult = await user_lookup(username);
    let message = transactionResult.result.data.message;
    if (message === 'New password required') {
        await submitNewPassword();
    }
    else if (message === 'Old password required') {
        showForm(login.forms.userNameExists);
    }
}

async function submitNewPassword() {
    const username = user.getUserName();
    const password = $(login.elements.newPassword).val();
    const verify = $(login.elements.verifyPassword).val();
    if (password && password === verify) {
        let transactionResult = await user_login(username, password);
        if (login.options.handleGameState) {
            gamestate.initializeGameLogDataForNewUser();
        }
        showForm(login.forms.success);
        if (loginCompleteCallback !== null) {
            loginCompleteCallback();
        }
    }
    else {
        showForm(login.forms.passwordMismatch);
    }
}

async function retryDialog(autoQuit = true) {
    // If autoQuit is true (default) the dialog will close the current window if the user chooses quit
    // Otherwise the result is returned and the caller is responsible for taking action
    let retryChoice = await dialog.showChoiceDialogAsync("A connection to the Internet could not be made. Do you want to try again?",
        ["Retry", "Quit"]);
    if (autoQuit && retryChoice === "Quit") {
        window.close();
    }
    return retryChoice;
}

function showPassword(password) {
    let fieldType = $(password).attr("type");
    let passwordElement = $(password);
    if (passwordElement.attr("type") === "password") {
        passwordElement.attr("type", "text");
    } else {
        passwordElement.attr("type", "password");
    }
}
$(async function() {
    login = await setConfigHandler.getConfigAsync();

    $("#greeting").text(login.options.greeting);

    home.initializeHomeDirectory();

    offline.setOfflinePlay(false); // Must be explicitly set to true if offline play is chosen

    $(login.elements.tryAgainButton).click({showForm:login.forms.initial}, showFormWithEventData);

    $(login.elements.tryNewAgainButton).click({showForm:login.forms.newUser}, showFormWithEventData);


    $(login.elements.submitNewUserAndPassword).click(submitNewUserAndPassword);

    $(login.elements.submitUserAndPassword).click(submitUserAndPassword);

    $(login.elements.newPlayerBtn).click({showForm: login.forms.newUser}, showFormWithEventData);


    // NoSuchName div buttons
    $(login.elements.noSuchNameBtn).click({showForm: login.forms.initial}, showFormWithEventData);

    // WrongPassword div buttons
    $(login.forms.wrongPassword).find("button").click({showForm: login.forms.initial}, clearPasswordFields);

    // success_form div link
    $(login.forms.success).find("a").click(milestone.goToCurrentMilestone);

    // Continue to play game
    $(login.elements.milestoneBtn).click(milestone.goToCurrentMilestone);
    // Show passwords
    $(login.elements.showOldPassword).click(() => showPassword(login.elements.oldPassword));

    $(login.elements.showNewPasswords).click(() => {
        showPassword(login.elements.newPassword);
        showPassword(login.elements.verifyPassword);
    });

    // Play offline
    $(login.elements.playOfflineBtn).click(playOffline);
    $(login.elements.offlineNewBtn).click(offlineNewGame);
    $(login.elements.offlineReturningBtn).click(offlineReturningPlayer);


    let searchParams = new URLSearchParams(window.location.search);
    if (searchParams.has("sign_off")) {
        // then we were previously logged in; remove user data from local storage
        user.resetUserName();
        if (login.options.handleGameState) {
            gamestate.initializeGameLogDataForNewUser();
        }
    }

});

export {setConfig, setLoginCompleteCallback};