// login.js
// module

import * as login     from "./login_v1_config.js";
import * as user      from "./username.js";
import * as db        from "./database_access.js";
import * as gamestate from "./gamestate.js";
import * as offline   from "./offline_play.js";
import * as milestone from "./milestone.js";
import * as home      from "./home.js";
import * as errorReport from "./errorReport.js";

let newuser= false ;
let loginCompleteCallback = null;

function setLoginCompleteCallback(f) {
    loginCompleteCallback = f;
}

function showForm(formID) {
    let forms = Object.entries(login.forms);
    for (const [, ID] of forms) {
        if (ID === formID) {
            $(ID).show();
        }
        else {
            $(ID).hide();
        }
    }
}

function showFormWithEventData(evt) {
    if (evt.data && evt.data.showForm) {showForm(evt.data.showForm);}
    evt.preventDefault();
}

function user_lookup(theUser, completionRoutine){
    let dbPromise = db.perform_db_action(
        {
            action:'lookup-userid',
            userid: theUser
        }
    );

    if (typeof(completionRoutine) === "function") {
        dbPromise.done(completionRoutine);
    }

    dbPromise.fail(() => retryDialog(() => user_lookup(theUser, completionRoutine)));
}

function user_login(theUser, password, completionRoutine){
    let dbPromise = db.perform_db_action(
        {
            action:'user-login',
            userid: theUser,
            password: password
        }
    );

    if (typeof(completionRoutine) === "function") {
        dbPromise.done(completionRoutine);
    }

    dbPromise.fail(() => retryDialog(() => user_login(theUser, completionRoutine)));
}

// Looks up username. Gives error message if doesn't exist on start page ;
// Gives error message if duplicate username on new user page ;
function finishLookupUserName(data) {
    if (data.message === 'New password required') {
        if (newuser) {
            submitNewPassword();
        }
        else {
            showForm(login.forms.nameNotFound);
        }
    }
    else if (data.message === 'Old password required' && newuser) {
        showForm(login.forms.userNameExists);
    }
    else {
        submitExistingPassword();
    }

}

function submitNewPassword() {
    const username = user.getUserName();
    const password = $(login.elements.newPassword).val();
    const verify = $(login.elements.verifyPassword).val();
    if (password && password === verify) {
        user_login(username, password, finishSubmitNewPassword);
    }
    else {
        showForm(login.forms.passwordMismatch);
    }
}

function submitExistingPassword() {
    let username = user.getUserName();
    let password = $(login.elements.oldPassword).val();
    if (password) {
        user_login(username, password, finishSubmitExistingPassword);
    }
}

function finishSubmitNewPassword(data) {
    gamestate.initializeGameLogDataForNewUser();
    showForm(login.forms.success);
    if (loginCompleteCallback !== null) {
        loginCompleteCallback();
    }
}

function extractGameStateData(data) {
    // Save JSON for gamestate in session storage,
    // and also extract some game state components.
    let jsondata = data.jsondata;
    if (jsondata) {
        gamestate.setGameStateFromJSON(jsondata);
    }
    else {
        gamestate.setGameStateFromJSON("");
    }

    if ("points" in data && data.points != null) {
        gamestate.setPoints(data.points);
    }
    else {
        gamestate.setPoints(0);
    }

    if ("session_count" in data && data.session_count != null) {
        gamestate.setSession(data.session_count);
    }
    else {
        gamestate.setSession(1);
    }
}

function finishSubmitExistingPassword(data) {
    if (!data || data.message === "Password did not match") {
        showForm(login.forms.wrongPassword)
    }
    else {
        extractGameStateData(data);
        showForm(login.forms.success);
        if (loginCompleteCallback !== null) {
            loginCompleteCallback();
        }
    }
}

function playOffline(){
    showForm(login.forms.offline);
    return false;
}

function offineNewGame() {
    offline.setOfflinePlay();
    gamestate.setGameStateFromJSON("{}");
    showForm(login.forms.success);
    return false;
}

function offineReturningPlayer() {
    offline.setOfflinePlay();
    var jsondata = localStorage.getItem(localGameStateKey);
    extractGameStateData(jsondata);
    showForm(login.forms.success);
    return false;
}

function clearUserNameFields() {
    $(login.elements.oldUsername).val("");
    $(login.elements.newUsername).val("");
}

function clearPasswordFields(args) {
    $(login.elements.newPassword).val("");
    $(login.elements.verifyPassword).val("");
    $(login.elements.oldPassword).val("");
    showForm(args.data.showForm);
}

function refreshLoginForm() {
    clearUserNameFields()
    clearPasswordFields();
    showForm(login.forms.initial);
    return false;
}


function submitUserAndPassword(evt) {
    newuser = false;
    const username = $(login.elements.oldUsername).val();
    user.setUserName(username);
    user_lookup(username, finishLookupUserName);
    evt.preventDefault();
}

function submitNewUserAndPassword(evt) {
    newuser = true;
    const username = $(login.elements.newUsername).val();
    user.setUserName(username);
    user_lookup(username, finishLookupUserName);
    evt.preventDefault();
}

function retryDialog(retryFunction) {
    errorReport.showDialog({
        header: "Connection Error",
        message: "A connection to the Internet could not be made. Do you want to try again?",
        buttons: {
            Retry: function() {
                retryFunction();
                $(".my-dialog").remove();
            },
            Quit: function() {
                window.close();
            }
        }
    });
}

function showPassword(password) {
    let fieldType = $(password).attr("type");
    let passwordElement = $(password);
    if (passwordElement.attr("type") === "password") {
        passwordElement.attr("type", "text");
    } else {
        passwordElement.attr("type", "password");
    }
}
$(function() {
    home.initializeHomeDirectory();

    offline.setOfflinePlay(false); // Must be explicitly set to true if offline play is chosen

    $(login.elements.tryAgainButton).click({showForm:login.forms.initial}, showFormWithEventData);

    $(login.elements.tryNewAgainButton).click({showForm:login.forms.newUser}, showFormWithEventData);


    $(login.elements.submitNewUserAndPassword).click(submitNewUserAndPassword);

    $(login.elements.submitUserAndPassword).click(submitUserAndPassword);

    $(login.elements.newPlayerBtn).click({showForm: login.forms.newUser}, showFormWithEventData);


    // NoSuchName div buttons
    $(login.elements.noSuchNameBtn).click({showForm: login.forms.initial}, showFormWithEventData);

    // WrongPassword div buttons
    $(login.forms.wrongPassword).find("button").click({showForm: login.forms.initial}, clearPasswordFields);

    // success_form div link
    $(login.forms.success).find("a").click(milestone.goToCurrentMilestone);

    // Continue to play game
    $(login.elements.milestoneBtn).click(milestone.goToCurrentMilestone);
    // Show passwords
    $(login.elements.showOldPassword).click(() => showPassword(login.elements.oldPassword));

    $(login.elements.showNewPasswords).click(() => {
        showPassword(login.elements.newPassword);
        showPassword(login.elements.verifyPassword);
    });

    // Play offline
    $(login.elements.playOfflineBtn).click(playOffline);
    $(login.elements.offlineNewBtn).click(offineNewGame);
    $(login.elements.offlineReturningBtn).click(offineReturningPlayer);


    let searchParams = new URLSearchParams(window.location.search);
    if (searchParams.has("sign_off")) {
        // then we were previously logged in; remove user data from local storage
        user.resetUserName();
        gamestate.initializeGameLogDataForNewUser();
    }

});

export {setLoginCompleteCallback};